/* 
 * automatic image capture without window
 *
 * modified from cap_window.c
 * 2004.11.22 -
 * Time-stamp: <2005-01-27 19:37:46 dimm>
 *
 *
 * captured image is stored in guchar* graybuf
 *  as a 8-bit gray-scale image.
 *  width and height are in int wid, hgt
 *
 */
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "finder.h"
#include "environ.h"

GdkPixbuf *pixbuf;

/* functions in this file */
//int auto_start_capture(void);
//int auto_stop_capture(void);
//void timeout_reload_auto(gpointer pointer);


/* start capture timer */
int auto_start_capture(void)
{
  if (timertag == -1) {
    /* start timer */
    printf("setting timer interval %d msec.\n", mseemon.time_reload);
    if (fout != NULL) {
      output_ctime(fout);
      fprintf(fout, "setting timer interval %d msec.\n", mseemon.time_reload);
    }

    timertag = gtk_timeout_add(mseemon.time_reload, (GtkFunction)timeout_reload_auto, NULL);

    #ifdef DEBUG
    printf("timertag is %d.\n", timertag);
    #endif
    //    mseemon.is_capturing = 1;
  }
}

/* stop capturing */
void auto_stop_capture (void) 
{
  gtk_timeout_remove(timertag);
  timertag = -1;
  //  mseemon.is_capturing = 0;

  //  gtk_main_quit ();

  /* clean up position file */
  mseemon.star_center[0].cx = -1;
  mseemon.star_center[0].cy = -1;
  write_finder_coords(); /* in socket_client.c */

}



void timeout_reload_auto(gpointer pointer)
{
  int i, ret, errflg = 0;
  //  gchar str[50];

  #ifdef DEBUG
  printf("# timeout_reload_auto: timer was called.\n");
  #endif

  /* check previous reload is finished */
  if (mseemon.flg_reload) {
    printf("### WARNING: timer overflow\n");
    errflg = 1;
    prg_cnt++;
    if (prg_cnt > MAX_ERROR) {
      /* terminate with error code */
      exit(EXIT_ERROR);
    }
    return;
  }

  /* turn on reloading flg */
  mseemon.flg_reload = 1;

  /* dome status */
  domestatus = read_dome_status(); /* in socket_client.c v0.7.0 20041123 */

  dvgrab(); /* capture from dv */

  if (!(load_pix_buffer())) { /* load to pixbuf */
    printf("# error in reading from capture image.\n");

    errflg = 1;
    prg_cnt++;
    if (prg_cnt > MAX_ERROR) {
      /* terminate with error code */
      exit(EXIT_ERROR);
    }

    return;
  }

  if ((copy_to_gray_buffer())) {
    /* output coords to fifo */
    if (pos_sw) {
      ret = get_star(); /* in find_star.c */
      if (ret) { /* succeeded */

	/* calculate background value and sigma */
	ret = get_bg();

	/* search peak */
	ret = search_peak(0);
	if (ret == 1) {
	  #ifdef DEBUG
	  printf("# search_peak succeeded. ");
	  printf("sky:%d sig:%5.2f peak:%dx%d\n", mseemon.skyval, mseemon.skysig, mseemon.star_center[0].cx, mseemon.star_center[0].cy);
	  #endif
	  mseemon.star_center[0].cx = mseemon.peak[0].cx;
	  mseemon.star_center[0].cy = mseemon.peak[0].cy;
	  starpos.cx = mseemon.peak[0].cx;
	  starpos.cy = mseemon.peak[0].cy;
	  is_star_found = 1;

	  #ifdef DEBUG
	  printf("%d %d %d %d %d %d\n", starpos.cx, starpos.cy, mseemon.peakval[0], rainflg, humidity, domestatus);
	  #endif
	  //	  write_coord();
	  write_finder_coords(); /* in socket_client.c */
	} else {
	  printf("# search peak failed.\n");
	  if (fout != NULL) {
	    output_ctime(fout);
	    fprintf(fout, "# search peak failed.\n");
	  }
	  
	  is_star_found = 0;
	}
      }
    }

    if (mseemon.is_capturing) { /* cap window is open */
      #ifdef DEBUG
      printf("# redraw_image called.\n");
      #endif
      redraw_image(); /* in cap_window.c */
    }
  }

  if (domestatus == 1) {
    printf("# dome is closed. stop operation. \n");
    if (fout != NULL) {
      output_ctime(fout);
      fprintf(fout, "# dome is closed. stop operation. \n");
    }
    auto_stop_capture();
  }

  /* turn off reloading flg */
  mseemon.flg_reload = 0;

  if (errflg == 0) {
    /* no error. reset counter */
    prg_cnt = 0;
  }

  //  if (fout != NULL) {
  //    fflush(fout);
  //  }
  #ifdef DEBUG
  printf("# timeout_reload_auto finished.\n");
  #endif

}

