/* 
 * image capture window
 * and capturing from DV
 *
 * 2002.10.26 -
 * for finder 2003.03.11 -
 * Time-stamp: <2005-01-27 19:39:20 dimm>
 *
 *
 * captured image is stored in guchar* graybuf
 *  as a 8-bit gray-scale image.
 *  width and height are in int wid, hgt
 *
 */
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "finder.h"
#include "environ.h"

GdkPixbuf *pixbuf;

/* functions in this file */
void dvgrab(void);              /* image read from DV */
int load_pix_buffer(void);      /* load pix buffer */
int copy_to_gray_buffer(void);  /* copy from pixbuf to gray buffer */
int init_cap_window(GtkWidget *window_cap); /* initialization of capture window */
void redraw_image (void);       /* redraw image */
//int write_coord();              /* output coords to file v0.5a2 */


/* callbacks */
gboolean on_darea_expose (GtkWidget *widget, GdkEventExpose *event, gpointer user_data); /* callback for expose event */
int on_click_darea (GtkWidget *widget, GdkEventButton *event);    /* callback for a mouse click */
void callback_reload_image(GtkWidget *widget, gpointer pointer);  /* reload button */
void timeout_reload_image(gpointer pointer); /* callback for timeout reload */
void callback_open_window_cap (GtkWidget *widget, gpointer data); /* callback for opening capture window */
void callback_hide_window_cap (GtkWidget *widget, gpointer data); /* callback for hiding capture window */
void callback_toggle_coord_out(GtkWidget *widget, gpointer pointer); /* callback for coord_output cntrl button. v0.5 */


/* image read from DV
 *
 * NOTE: this is preliminary [2002.11.29]
 */
void dvgrab(void){
  pid_t new_pid, child_pid;
  int stat_val;
 
  /* dvgrab */
  //  system("/usr/local/bin/grab_single2.sh");

  /* usign fork(); v0.6.3 */
  system("/usr/local/bin/grab");
}

/* load pix buffer */
int load_pix_buffer(void) {

  /* release and destroy previous pixbuf */
  if (pixbuf != NULL) {
    g_object_unref(pixbuf);
  }

  /* load from file
   *  preliminary.
   *  read from .ppm file made by "transcode" command.
   */  
  //  #ifdef DEBUG
  //  printf("PPM: %s\n", mseemon.dvfilepath);
  //  #endif
  if ((pixbuf = gdk_pixbuf_new_from_file(mseemon.dvfilepath, NULL)) == NULL) {
    printf("failed opening ppm file.\n");
    if (fout != NULL) {
      output_ctime(fout);
      fprintf(fout, "failed opening ppm file.\n");
    }
    return(0);
  }

  /* commented out in 2003.02.11. this seems to cause
   * memory leak */ 
  /*  g_object_ref(pixbuf); /* add reference */
  wid = gdk_pixbuf_get_width(pixbuf);
  hgt = gdk_pixbuf_get_height(pixbuf);

  return(1);
}


/* copy from pixbuf 
 * to gray buffer
 */
int copy_to_gray_buffer(void) {
  int i, j;
  guchar *inpix, *outpix;

  if ((inpix = gdk_pixbuf_get_pixels(pixbuf)) == NULL) {
    printf("error in reading pixbuf.\n");
    return(0);
  }

  outpix = graybuf;

  for (i = 0; i < hgt; i++) {
    for (j = 0; j < wid; j++) {
      *outpix = *inpix;
      inpix += 3;
      outpix ++;
    }
  }

  #ifdef DEBUG
  printf("finished copying to gray buffer.\n");
  #endif

  return(1);
}

 
/* redraw image */
void redraw_image (void) {
  int i;
  GtkWidget* w;
  int tx1, tx2, ty1, ty2;

  w = g_image_area1;
  gdk_draw_gray_image (GDK_DRAWABLE(w->window), w->style->fg_gc[GTK_STATE_NORMAL],
		       0, 0, wid, hgt,
		       GDK_RGB_DITHER_MAX, graybuf, wid);

  if (is_marked) {
    tx1 = mark.cx - 20;
    tx2 = mark.cx + 20;
    gdk_draw_line(GDK_DRAWABLE(w->window), w->style->white_gc, tx1, mark.cy, tx2, mark.cy);
    ty1 = mark.cy - 20;
    ty2 = mark.cy + 20;
    gdk_draw_line(GDK_DRAWABLE(w->window), w->style->white_gc, mark.cx, ty1, mark.cx, ty2);
  }

  if (is_star_found) {
    tx1 = starpos.cx - 2;
    tx2 = starpos.cx + 2;
    ty1 = starpos.cy - 2;
    gdk_draw_line(GDK_DRAWABLE(w->window), w->style->black_gc, tx1, ty1, tx2, ty1);
    tx1 = starpos.cx - 2;
    tx2 = starpos.cx + 2;
    ty1 = starpos.cy + 2;
    gdk_draw_line(GDK_DRAWABLE(w->window), w->style->black_gc, tx1, ty1, tx2, ty1);
    ty1 = starpos.cy - 2;
    ty2 = starpos.cy + 2;
    tx1 = starpos.cx + 2;
    gdk_draw_line(GDK_DRAWABLE(w->window), w->style->black_gc, tx1, ty1, tx1, ty2);
    ty1 = starpos.cy - 2;
    ty2 = starpos.cy + 2;
    tx1 = starpos.cx - 2;
    gdk_draw_line(GDK_DRAWABLE(w->window), w->style->black_gc, tx1, ty1, tx1, ty2);
  }

}

/* expose-event */
gboolean on_darea_expose (GtkWidget *widget, 
			 GdkEventExpose *event, 
			 gpointer user_data) {
  int i;

  #ifdef DEBUG
  printf("expose-event occured.\n");
  #endif

  if ((copy_to_gray_buffer())) {
    gdk_draw_gray_image (GDK_DRAWABLE(widget->window), widget->style->fg_gc[GTK_STATE_NORMAL],
			 0, 0, wid, hgt,
			 GDK_RGB_DITHER_MAX, graybuf, wid);
  } 

}


/* callback for a mouse click */
/* for finder. 2003.03.11 */
int on_click_darea (GtkWidget *widget, 
		    GdkEventButton *event) {
  gchar str[50];

  #ifdef DEBUG
  printf("button press event occured. button:%d", event->button);
  #endif

  if (event->button == 1) {
    /* left button is clicked */
    if (marker_mode) {
      mark.cx = (int)(event->x + 0.5);
      mark.cy = (int)(event->y + 0.5);
      is_marked = 1;
      marker_mode = 0;
    } else {
      mseemon.star_center[0].cx = (int)(event->x + 0.5);
      mseemon.star_center[0].cy = (int)(event->y + 0.5);
    }
#ifdef DEBUG
    printf("left: x:%d y:%d\n", mseemon.star_center[0].cx, mseemon.star_center[0].cy);
#endif
    sprintf(str, "x:%3d y:%3d",
	    mseemon.star_center[0].cx,
	    mseemon.star_center[0].cy);
    gtk_entry_set_text(GTK_ENTRY(ent_r0), str);
  } else if (event->button == 3) {
    /* right button is clicked */
    /* cancel mark */
    is_marked = 0;
    marker_mode = 0;
  }
  return(1);
}


/* callback for reload button */
void callback_reload_image(GtkWidget *widget, gpointer pointer)
{
  GtkWidget* w;

  w = pointer; /* pointer for g_image_area */

  dvgrab(); /* capture from dv */
  if (!(load_pix_buffer())) { /* load to pixbuf */
    printf("error in reading from capture image.\n");
  }
  if ((copy_to_gray_buffer())) {
    /* redraw image */
    redraw_image();
  }
}

/* callback for marker button */
void callback_marking(GtkWidget *widget, gpointer pointer)
{
  gchar str[50];
  GtkWidget* w;

  w = pointer; /* pointer for g_image_area */

  if (marker_mode == 0) {
    /* start marking */
    marker_mode = 1;
    sprintf(str, "start marking.");
    gtk_entry_set_text(GTK_ENTRY(ent_r0), str);
  } else {
    /* end marking */
    marker_mode = 0;
    is_marked = 0;
    sprintf(str, "end marking mode.");
    gtk_entry_set_text(GTK_ENTRY(ent_r0), str);
  }
}

/* callback for timeout reload */
void timeout_reload_image(gpointer pointer)
{
  int i, ret, errflg = 0;
  GtkWidget* w;
  gchar str[50];

  w = pointer; /* pointer for g_image_area */

  //  #ifdef DEBUG
  //  printf("timer is called.\n");
  //  #endif

  /* check previous reload is finished */
  if (mseemon.flg_reload) {
    printf("### WARNING: timer overflow\n");
    errflg = 1;
    prg_cnt++;
    if (prg_cnt > MAX_ERROR) {
      /* terminate with error code
       * v0.6.3 2004.02.22 */
      /* modified v0.6.7 20041104 */
      exit(EXIT_ERROR);
    }
    return;
  }

  /* turn on reloading flg */
  mseemon.flg_reload = 1;

  /* dome status */
  domestatus = read_dome_status(); /* in socket_client.c v0.7.0 20041123 */

  dvgrab(); /* capture from dv */

  if (!(load_pix_buffer())) { /* load to pixbuf */
    printf("# error in reading from capture image.\n");

    errflg = 1;
    prg_cnt++;
    if (prg_cnt > MAX_ERROR) {
      /* terminate with error code
       * v0.6.3 2004.02.22 */
      /* modifed v0.6.7 20041104 */
      exit(EXIT_ERROR);
    }

    /* rest for a while and try again */
    /* added v0.6.3 2004.02.21 */
    //    stoptimer();
    //    printf("# sleeping 10sec and try again.\n");
    //    sleep(10);
    //    recreate_cap_window();

    return;
  }

  if ((copy_to_gray_buffer())) {

    /* output coords to fifo */
    if (pos_sw) {
      ret = get_star(); /* in find_star.c */
      if (ret) { /* succeeded */

	/* calculate background value and sigma */
	ret = get_bg();

	/* search peak */
	ret = search_peak(0);
	if (ret == 1) {
	  #ifdef DEBUG
	  printf("# search_peak succeeded. ");
	  printf("sky:%d sig:%5.2f peak:%dx%d\n", mseemon.skyval, mseemon.skysig, mseemon.star_center[0].cx, mseemon.star_center[0].cy);
	  #endif
	  mseemon.star_center[0].cx = mseemon.peak[0].cx;
	  mseemon.star_center[0].cy = mseemon.peak[0].cy;
	  starpos.cx = mseemon.peak[0].cx;
	  starpos.cy = mseemon.peak[0].cy;
	  is_star_found = 1;

	  sprintf(str, "x:%3d y:%3d val:%3d mark:x:%3d y:%3d",
		  mseemon.star_center[0].cx,
		  mseemon.star_center[0].cy,
		  mseemon.peakval[0],
		  mark.cx, mark.cy);
	  gtk_entry_set_text(GTK_ENTRY(ent_r0), str);

	  //	  write_coord();
	  write_finder_coords(); /* in socket_client.c */
	} else {
	  printf("search peak failed.\n");
	  if (fout != NULL) {
	    output_ctime(fout);
	    fprintf(fout, "search peak failed.\n");
	  }
	  
	  is_star_found = 0;
	}
      }
    }

    /* redraw image */
    redraw_image();
    #ifdef DEBUG
    printf("finished redraw image.\n");
    #endif
    if (fout != NULL) {
      output_ctime(fout);
      fprintf(fout, " finished redraw image.\n");
    }
  }

  if (domestatus == 1) {
    printf("# dome is closed. stop operation. \n");
    if (fout != NULL) {
      output_ctime(fout);
      fprintf(fout, "# dome is closed. stop operation. \n");
    }
    callback_hide_window_cap (NULL, (gpointer)window_capture);
  }

  /* turn off reloading flg */
  mseemon.flg_reload = 0;

  if (errflg == 0) {
    /* no error. reset counter */
    prg_cnt = 0;
  }

  //  if (fout != NULL) {
  //    fflush(fout);
  //  }
}

/* deprecated v0.7.0 20041123 iwata */
/* output coords to file v0.5a2 */
/* v0.6.2 added rainflg and humidity */
//int write_coord(void) {
//  FILE *fp;
//  int tx, ty;

//  if ((fp=fopen(POSFILE, "w")) == NULL) {
//    printf("can't open %s.\n", POSFILE);
//    return(EXIT_FAILURE);
//  }

//  //  tx = IMG_WID/2 - mseemon.star_center[0].cx;
//  //  ty = IMG_HGT/2 - mseemon.star_center[0].cy;
//  tx = mseemon.star_center[0].cx;
//  ty = mseemon.star_center[0].cy;

//  //  fprintf(fp, "%d %d %d\n", tx, ty, mseemon.peakval[0]);
//  fprintf(fp, "%d %d %d %d %d %d\n", tx, ty, mseemon.peakval[0], rainflg, humidity, domestatus);

//  fclose(fp);
//}


/* start timer */
void starttimer(void) {
  printf("setting timer interval %d msec.\n", mseemon.time_reload);
  if (fout != NULL) {
    output_ctime(fout);
    fprintf(fout, "setting timer interval %d msec.\n", mseemon.time_reload);
  }

  timertag = gtk_timeout_add(mseemon.time_reload, (GtkFunction)timeout_reload_image, g_image_area1);

  #ifdef DEBUG
  printf("timertag is %d.\n", timertag);
  #endif

}

/* stop timer */
void stoptimer(void) {
  printf("stoptimer is called.\n");
  gtk_timeout_remove(timertag);
  timertag = -1;
}

/* callback: open capture window */
void callback_open_window_cap (GtkWidget *widget, gpointer data)
{
  gtk_widget_show((GtkWidget*)data);

  if (timertag == -1) {
    if (!mseemon.is_auto) {
      starttimer();
    }
  }
  /* capture flag */
  mseemon.is_capturing = 1;
}

/* callback for hide window */
void callback_hide_window_cap (GtkWidget *widget, gpointer data)
{
  gtk_widget_hide((GtkWidget*)data);
  if (!mseemon.is_auto) {
    stoptimer();
  }
  /* capture flag */
  mseemon.is_capturing = 0;
}

/* callback for button for control output */
void callback_toggle_coord_out(GtkWidget *widget, gpointer pointer)
{
  if (pos_sw == 1) {
    gtk_button_set_label(GTK_BUTTON(widget), "Start Output");
    pos_sw = 0;
    //    is_marked = 0;
  } else {
    gtk_button_set_label(GTK_BUTTON(widget), " Stop Output");
    pos_sw = 1;
  }
}


/* callback for find star pos. button */
void callback_find_starpos(GtkWidget *widget, gpointer pointer)
{
  int ret;
  gchar str[50];

  ret = get_star(); /* in find_star.c */
  if (ret) {
    /* succeeded */
    #ifdef DEBUG
    printf("# get_star succeeded.\n");
    #endif
    flg_set_rect[0] = 1;
    ret = search_peak(0);
    if (ret == 1) {
      #ifdef DEBUG
      printf("# search_peak succeeded.\n");
      #endif
      mseemon.star_center[0].cx = mseemon.peak[0].cx;
      mseemon.star_center[0].cy = mseemon.peak[0].cy;
      starpos.cx = mseemon.peak[0].cx;
      starpos.cy = mseemon.peak[0].cy;
      is_star_found = 1;

      redraw_image();
      sprintf(str, "x:%3d y:%3d",
	      mseemon.star_center[0].cx,
	      mseemon.star_center[0].cy);
      gtk_entry_set_text(GTK_ENTRY(ent_r0), str);
      if (fout != NULL) {
	output_ctime(fout);
	fprintf(fout, "center: %3d %3d %3d\n",
		mseemon.star_center[0].cx,
		mseemon.star_center[0].cy,
		mseemon.peakval[0]);
      }
    } else {
      printf("#search_peak failed.\n");
      is_star_found = 0;
    }
  } else {
    /* failed */
    flg_set_rect[0] = 0;
    printf("#find_star failed.\n");
    is_star_found = 0;
  }
}


/* initialization of capture window 
 *
 */
int init_cap_window(GtkWidget *window_cap) {
  int i;
  GtkWidget *button;
  GtkWidget *box_cap1, *box_cap2;

  /* g_image_area1 is global */

  box_cap1 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window_cap), box_cap1);

  /* load pix buffer and 
   * know the width and height of the image */
  pixbuf = NULL;
  if (!(load_pix_buffer())) {
    printf("error in reading from capture image.\n");
    return(0);
  }
  if ((graybuf = (guchar*)malloc(sizeof(unsigned char)*wid*hgt)) == NULL) {
    printf("failed in allocating memory for image buffer.\n");
    exit(EXIT_ERROR);
  }

  /* drawing area for capture window */
  gdk_rgb_init(); /* init of GdkRGB */
  g_image_area1 = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(g_image_area1), wid, hgt);
  gtk_signal_connect(GTK_OBJECT(g_image_area1), "expose-event", 
		     GTK_SIGNAL_FUNC(on_darea_expose), NULL);
  gtk_signal_connect(GTK_OBJECT(g_image_area1), "button_press_event", 
		     GTK_SIGNAL_FUNC(on_click_darea), NULL);
  gtk_widget_set_events(g_image_area1, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);

    
  #ifdef DEBUG
  printf("window_cap: %p\n", window_cap);
  printf("image_area: %p\n", g_image_area1);
  printf("pImRect0: %p, 1: %p\n", pImRect[0], pImRect[1]);
  #endif

  gtk_box_pack_start(GTK_BOX(box_cap1), g_image_area1, TRUE, TRUE, 0);
  gtk_widget_show(g_image_area1);

  /* output window */
  box_cap2 = gtk_hbox_new (FALSE, 0);
  ent_r0 = gtk_entry_new();
  gtk_entry_set_width_chars(GTK_ENTRY(ent_r0), 70);
  gtk_box_pack_start(GTK_BOX(box_cap2), ent_r0, TRUE, TRUE, 0);
  gtk_widget_show(ent_r0);
  gtk_box_pack_start(GTK_BOX(box_cap1), box_cap2, TRUE, TRUE, 0);
  gtk_widget_show (box_cap2);

  /* button for control fifo */
  box_cap2 = gtk_hbox_new (FALSE, 0);
  button = gtk_button_new_with_label ("Start Output");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_toggle_coord_out), g_image_area1);
  gtk_box_pack_start(GTK_BOX (box_cap2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);
  //  btn1 = button;

  /* reload button */
  /* hidden from v0.4 2003.08.16
  box_cap2 = gtk_hbox_new (FALSE, 0);
  button = gtk_button_new_with_label ("Reload");

  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_reload_image), g_image_area1);

  gtk_box_pack_start(GTK_BOX (box_cap2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);
  */

  /* star-find button */
  //  box_cap2 = gtk_hbox_new (FALSE, 0);
  button = gtk_button_new_with_label ("Find Star");

  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_find_starpos), g_image_area1);

  gtk_box_pack_start(GTK_BOX (box_cap2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  /* marker button */
  button = gtk_button_new_with_label ("Mark");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_marking), window_cap);
  gtk_box_pack_start(GTK_BOX (box_cap2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  /* close button for capture window */
  button = gtk_button_new_with_label ("Close");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_hide_window_cap), window_cap);


  gtk_box_pack_start(GTK_BOX (box_cap2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);
  gtk_box_pack_start(GTK_BOX(box_cap1), box_cap2, TRUE, TRUE, 0);
  gtk_widget_show (box_cap2);
  gtk_widget_show (box_cap1);


  return(1);
}


/* destroy and re-create capture window
 * v0.6.3 2004.02.21 */
int recreate_cap_window(void) {
  GtkWidget *window_cap;    /* capture window */

  /* destroy current capture window */
  gtk_widget_destroy(GTK_WIDGET(window_capture));

  window_cap = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  window_capture = window_cap;

  init_cap_window(window_cap);

  gtk_widget_show((GtkWidget*)window_cap);

  if (timertag == -1) {
    starttimer();
  }
  /* capture flag */
  mseemon.is_capturing = 1;

  return(EXIT_SUCCESS);

}
