/*
 * find_star.c : automatically find stars 
 *
 * 2003.02.11 -
 * 2003.08.16 - revision for finder
 * Time-stamp: <2004-11-03 00:46:15 dimm>
 */

#include <math.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "finder.h"

/* find a star in image
 * 2003.05.29 update
 * 2003.08.17 for finder
 */
int get_star(void) {
  int tx, ty, sx, sy, i, j, k, cx, cy;
  int flg, cx2;
  int maxx, maxy, maxval;
  //  int maxx2, maxy2, maxval2;
  int average;
  guchar *pix;
  pixcoord swp;

  /* steps */
  sx = 30;
  sy = 30;
  /* cell size */
  tx = mseemon.star_rect[0].cx;
  ty = mseemon.star_rect[0].cy;

  pix = graybuf;

  #ifdef DEBUG
  printf("now start finding.\n");
  #endif

  maxval = -1;
  
  /* sum values of pixels in each cell
   * and find a cell with highest value
   */
  cx = tx;
  cy = ty;
  flg = 1;

  while (flg) {
    average = 0;
    for (i = 0; i < ty; i++) {
      for (j = 0; j < tx; j++) {
	average += pix[(cy + i) * wid + cx + j];
      }
    }

    if (average > maxval) {
      maxx = cx;
      maxy = cy;
      maxval = average;
    }

    /* increment */
    cx += sx;
    if (cx > wid - tx) {
      cx = 0;
      cy += sy;
      if (cy > hgt - ty) {
	flg = 0;
      }
    }
  }

  #ifdef DEBUG
  printf("cell w max value: %3d,%3d %d\n", maxx, maxy, maxval);
  #endif

  /* set star positions */ 
  mseemon.star_center[0].cx = maxx + tx/2;
  mseemon.star_center[0].cy = maxy + ty/2;

  /* success */
  return(1);
}


/*********************************
 * search_peak
 * 2003.05.02
 *********************************/
int search_peak(int n)
{
  int max, maxx, maxy;
  int smooth, smhalf, sum;
  int i, j, rx, ry;

  //  #ifdef DEBUG
  //  printf("search_peak called. for star %d\n", n);
  //  #endif

  smooth = SMOOTH_SIZE;
  smhalf = (int)((smooth -1)/2);

  /* check graybuf */
  if (graybuf == NULL) {
    return(-1);
  }

  /* check rects */
  //  if (pImRect[n] == NULL) {
  //    printf("#ERROR in search_peak: pImRect is NULL.\n");
  //    return(-1);
  //  }

  max = 0;

  for ( i = smhalf; i < mseemon.star_rect[n].cy - smhalf; i++ ) {
    for ( j = smhalf; j < mseemon.star_rect[n].cx - smhalf; j++ ) {
      rx = mseemon.star_center[n].cx + j - mseemon.star_rect[n].cx / 2;
      ry = mseemon.star_center[n].cy + i - mseemon.star_rect[n].cy / 2;
      if (ry < hgt-1 && ry >= 1 && rx < wid-1 && rx >= 1) {
	/* smoothed pixel count */
	sum = graybuf[ (ry-1) * wid + rx - 1] 
	  + graybuf[ (ry -1) * wid + rx]
	  + graybuf[ (ry -1) * wid + rx + 1]
	  + graybuf[ ry * wid + rx - 1]
	  + graybuf[ ry * wid + rx]
	  + graybuf[ ry * wid + rx + 1]
	  + graybuf[ (ry + 1) * wid + rx - 1]
	  + graybuf[ (ry + 1) * wid + rx]
	  + graybuf[ (ry + 1) * wid + rx + 1];
	if (sum > max) {
	  max = sum;
	  maxx = rx;
	  maxy = ry;
	}
      }
    }
  }

  mseemon.peak[n].cx = maxx;
  mseemon.peak[n].cy = maxy;

  mseemon.peakval[n] = graybuf[ maxy * wid + maxx];

  /* sufficiently higher than sky ? */
  if (mseemon.skyval > 0) {
    if (mseemon.peakval[n] > mseemon.skyval + (int)(3. * mseemon.skysig)) {
      return(1);
    } else {
      return(0);
    }
  } else {
    return(0);
  }
}


/* set background value and stddev
 * 2004.10.19 iwata
 */
int get_bg(void) {
  int i, j, k, k2;
  int step;
  float tot, mean, dev;
  guchar bgarray1[3072], bgarray2[3072];
  guchar t;

  step = 10;
  k = 0;

  for (i = step; i < hgt; i+=step) {
    for (j = step; j < wid; j+=step) {
      bgarray1[k] = graybuf[ i * wid + j];
      k++;
      if (k >= 3072) {
	i = hgt; j = wid;
      }
    }
  }


  for (i = 0; i < k; i++) {
    for (j = k-1; j > i; j--) {
      if (bgarray1[j] < bgarray1[j-1]) {
	t = bgarray1[j-1];
	bgarray1[j-1] = bgarray1[j];
	bgarray1[j] = t;
      }
    }
  }

  /* remove 10% upper and lower values */
  j = (int)(k * 0.1);
  k2 = 0;
  for (i = j; i < k - j; i++) {
    bgarray2[k2] = bgarray1[i];
    k2++;
  }
  k = k2;

  tot = 0.;
  for (i = 0; i < k; i++) {
    tot += bgarray2[i];
  }
  mean = tot / k;

  tot = 0.;
  for (i = 0; i < k; i++) {
    dev = bgarray2[i] - mean;
    tot += dev*dev;
  }

  dev = sqrt(tot/k);

  mseemon.skyval = (int)mean;
  mseemon.skysig = dev;

  return(1);

}
