/* 
 * finder.c
 * watec video cam caputure software
 *
 * based on seemon2. 2002.10.18 -
 * 2003.03.11 -
 * 
 * Time-stamp: <2005-01-27 19:39:41 dimm>
*/
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <string.h>
#include <fcntl.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include "finder.h"
#include "environ.h"

int main( int   argc,
          char *argv[] )
{
  int i, opt, ret;
  Obj *cobj;
  char key[10];

  /* GtkWidget is the storage type for widgets */
  GtkWidget *window;
  GtkWidget *window_cap;    /* capture window */
  GtkWidget *window_telctl; /* telescope control */
  GtkWidget *menubar;

  GtkWidget *button, *frame, *label;
  GtkWidget *vbox1, *vbox2;
  GtkWidget *hbox1, *hbox2;
  GtkWidget *entry;

  /* This is called in all GTK applications. Arguments are parsed
   * from the command line and are returned to the application. */
  gtk_init (&argc, &argv);

  /* set defaults */
  /* program status */
  prg_status = 1;
  prg_cnt = 0;

  /* timer for reload image */
  mseemon.time_reload = DEF_TIMER;
  timertag = -1;
  timertag_comm = -1;
  /* interval for setting sky */
  mseemon.time_sky = 5;
  mseemon.cnt_sky = 0;
  /* path of .ppm file */
  strcpy(mseemon.dvfilepath, PPMPATH);
  /* not connected, not tracking, not recording */
  //  mseemon.tel_stat = 0;
  mseemon.is_capturing = 0;
  mseemon.is_tracking = 0;
  mseemon.is_recording = 0;
  mseemon.motionflg = 0;
  mseemon.image_dist = 50.;
  /* flag for reloading */
  mseemon.flg_reload = 0;

  //  mseemon.loopflg = 0;
  mseemon.execflg = 1;

  /* flag indicating restarting or not */
  /* added v0.6.8 20041106 iwata */
  mseemon.is_restart = 0;
  mseemon.is_auto = 0;
  while ((opt = getopt(argc, argv, "ar")) != -1) {
    switch (opt) {
    case 'r':
      mseemon.is_restart = 1;
      printf("# restarting finder.\n");
      break;
    case 'a':
      mseemon.is_auto = 1;
      //      mseemon.loopflg = 1;
      printf("# auto-starting mode.\n");
      break;
    }
  }

  /* sky value and sigma */
  mseemon.skyval = -1;
  mseemon.skysig = -1;

  /* setup site info */
  /* oao */
  msite.slon = 133.59667; /* 8h 54m 23.2s, +34d 34m 26s */
  msite.slat = 34.573888; /* corrected on v.0.5.6 */
  msite.lt = 9;

  /* init for coord output */
  pos_sw = 0;
  is_star_found = 0;

  is_marked = 0;

  /* default rectangular image buffer */
  for (i=0; i < 2; i++) {
    mseemon.star_rect[i].cx = RECT_WID;
    mseemon.star_rect[i].cy = RECT_HGT;
    pImRect[i] = NULL;
    flg_set_rect[i] = 0;
    mseemon.cg[i].x = 0.;
    mseemon.cg[i].y = 0.;
    mseemon.peakval[i] = 0;
    mseemon.star_center[i].cx = 0;
    mseemon.star_center[i].cy = 0;
  }

  /* init variables in get_cg.c */
  bgthr = 1.; /* default background threshold */
  cgfactor = 1.;
  n_calc_r0 = 1;
  /* scale of one pixel in radian. */
  /* NOTE: these are for Celestron (2002.08.26). 
   * to be checked and updated. */
  //  scalex = PIXSCALE * 3.141592654 / (180*60*60);
  //  scaley = PIXSCALE * 3.141592654 / (180*60*60);
  //  Sigma.x = 0.;
  //  Sigma.y = 0.;
  //  Lambda = LAMBDA;
  NSample = 50;

  /* intialize log file */
  setoutputfile();

  /* socket connection */
  fd_socket = -1;
  if ((ret = connect_socket(SOCKET_SERVER)) == EXIT_SUCCESS) {
    printf("connected to %s.\n", SOCKET_SERVER);
  } else {
    printf("unable to connect to %s", SOCKET_SERVER);
    fd_socket=-1;
    //    mydialog(str);
  }

  /* check dome status */
  domestatus = read_dome_status(); /* in socket_client.c */

  /* check raindrop */
  rainflg = chk_raindrop();
  if (rainflg == RAIN_TRUE) {
    /* it is rainny. warning */
    mydialog("It is rainny.");
  } else if (rainflg == ENV_ERR) {
    mydialog("Raindrop Check Failed.");
  } else {
    printf("# raindrpchk: it is not rainny.\n");
  }

  /* check humidity */
  humidity = chk_humidity();
  printf("# current humidity: %d\n", humidity);
  if (humidity >= HUM_LIMIT) {
    mydialog("High Humidity.");
  }

  /* wait for a while */
  sleep(2);

  /* tell to finder_comm */
  #ifdef DEBUG
  printf("# telling current condition to finder_comm.\n");
  #endif
  write_finder_coords();

  /* init env-chk counter */
  //  env_cnt = 0;

  /* Create windows */
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  window_cap = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  /* store in a global variable */
  window_capture = window_cap;
  menubar = init_main_menu(window);
    
  /* set titles */
  gtk_window_set_title (GTK_WINDOW (window), "Finder: Main");
  gtk_window_set_title (GTK_WINDOW (window_cap), "Finder: Capture");

  /* Here we just set a handler for delete_event that immediately
   * exits GTK. */
  g_signal_connect (G_OBJECT (window), "delete_event",
		    G_CALLBACK (delete_event), NULL);

  /* Sets the border width of the window. */
  gtk_container_set_border_width (GTK_CONTAINER (window), 2);
  gtk_container_set_border_width (GTK_CONTAINER (window_cap), 10);
//  gtk_container_set_border_width (GTK_CONTAINER (window_telctl), 10);

  /*****
   * Making contents of Main Window
   *****/
  vbox1 = gtk_vbox_new (FALSE, 5);
  vbox2 = gtk_vbox_new (FALSE, 0);
  hbox1 = gtk_hbox_new (FALSE, 0);

  /* add menubar */
  gtk_box_pack_start (GTK_BOX(vbox1), menubar, FALSE, TRUE, 0);
  gtk_widget_show(menubar);

  gtk_container_add (GTK_CONTAINER (window), vbox1);
  //  gtk_box_pack_start (GTK_BOX(vbox1), vbox2, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(vbox1), hbox1, TRUE, TRUE, 0);


  /* buttons for dome */
  button = gtk_button_new_with_label("dome status");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_dome_status), NULL);
  gtk_box_pack_start (GTK_BOX(hbox1), button, TRUE, TRUE, 0);
  gtk_widget_show(button);

  button = gtk_button_new_with_label("open");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_dome_open), NULL);
  gtk_box_pack_start (GTK_BOX(hbox1), button, TRUE, TRUE, 0);
  gtk_widget_show(button);

  button = gtk_button_new_with_label("close");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_dome_close), NULL);
  gtk_box_pack_start (GTK_BOX(hbox1), button, TRUE, TRUE, 0);
  gtk_widget_show(button);

  gtk_widget_show (hbox1);


  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX(vbox1), hbox2, TRUE, TRUE, 0);

  /* button for capture window */
  button = gtk_button_new_with_label ("capture window");
  /* Now when the button is clicked, we call the "callback" function
   * with a pointer to "button 1" as its argument */
  // g_print("%p\n", window_cap); // debug
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_open_window_cap), (gpointer) window_capture);

  /* Instead of gtk_container_add, we pack this button into the invisible
   * box, which has been packed into the window. */
  gtk_box_pack_start (GTK_BOX(hbox2), button, TRUE, TRUE, 0);

  gtk_widget_show (button);
    
  /* Do these same steps again to create a second button */
  button = gtk_button_new_with_label ("quit");

  /* Call the same callback function with a different argument,
   * passing a pointer to "button 2" instead. */
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (delete_event), NULL);
  
  gtk_box_pack_start(GTK_BOX (hbox2), button, TRUE, TRUE, 0);
  gtk_widget_show(button);
  gtk_widget_show (hbox2);

  gtk_widget_show (vbox1);
  gtk_widget_show (window);


  /* making contents of capture window */
  init_cap_window(window_cap);

  printf("now starting finder..\n");

  /* if restarting, show up capture window 
   * v0.6.8 20041106 iwata */
  if (mseemon.is_restart) {
    callback_open_window_cap(NULL, (gpointer)window_cap);
    pos_sw = 1; /* start coord output */
  }


  /* start command check timer */
  //  if (mseemon.loopflg) {
  start_comm_timer(); /* in command.c */
  //  }

  //  do {
  //  if (mseemon.execflg) {
  //    mseemon.execflg = 0;

  /* *****************************
   * ****** auto-start mode ******
   *
   * if auto-start mode, check humidity, rainflg
   * then open the dome and start finding a star 
   * v0.7.0 20041122 iwata */
  if (mseemon.is_auto) {
    if (domestatus == 1) {
      if (rainflg == RAIN_FALSE &&
	  humidity > 0 && humidity < HUM_LIMIT) {
	/* environment check passed. open the dome. */
	if (fout != NULL) {
	  output_ctime(fout);
	  fprintf(fout, " opening the dome.\n");
	}

	sleep(1);

	#ifdef DEBUG
	printf("# opening the dome.\n");
	#endif
	dome_open();
	#ifdef DEBUG
	printf("# waiting 15 sec.\n");
	#endif
	sleep(15);
      } else {
	printf("# env. status: dome:%d ", domestatus);
	printf("rain:%d ", rainflg);
	printf("humidity: %d\n", humidity);
	printf("# canceled opening the dome.\n");
	if (fout != NULL) {
	  fprintf(fout, "# env. status: dome:%d ", domestatus);
	  fprintf(fout, "rain:%d ", rainflg);
	  fprintf(fout, "humidity: %d\n", humidity);
	  fprintf(fout, "# canceled opening the dome.\n");
	}
      }
    }

    /* check dome status */
    domestatus = read_dome_status(); /* in socket_client.c */
    /* update pos file */
    write_finder_coords(); /* in socket_client.c */

    if (domestatus == 3) {
      /* AC on */
      ret = ccd_on();
      if (ret == DOME_ERR) {
	exit(EXIT_ERROR);
      }

      /* wait 15 sec for secure start-up */
      printf("# wainting 15 sec.\n");
      sleep(15);

      auto_start_capture(); /* in cap_auto.c */
      pos_sw = 1; /* start coord output */
    }
  }

  /* Rest in gtk_main and wait for the fun to begin! */
  gtk_main ();

  /* close log file */
  closeoutputfile();
  //}
  //} while (mseemon.loopflg);


  /* tell finder_comm to quit */
  if (fd_socket >= 0) {
    printf("# sending finder_comm a quit signal.\n");
    if (fout != NULL) {
      fprintf(fout, "# sending finder_comm a quit signal.\n");
    }
    strcpy(key, "quit");
    write(fd_socket, key, sizeof(key));

    sleep(1);

    ret = disconnect_socket();
    if (ret != EXIT_SUCCESS) {
      printf("# error in disconnecting socket.\n");
      if (fout != NULL) {
	fprintf(fout, "# error in disconnecting socket.\n");
      }
    }
  }

  /* normal termination */
  /*  modifed v0.6.7 20041104 */
  return(EXIT_NORMAL);
}

